/*
    Z80 emulator

    Copyright (c) 1996-2003,2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "z80.h"

Z80::OpcodeInfo Z80::OpInfo_[256] = {
    { &Z80::opcode_00,  4 }, // NOP
    { &Z80::opcode_01, 10 }, // LD   BC,nn
    { &Z80::opcode_02,  7 }, // LD   (BC),A
    { &Z80::opcode_03,  6 }, // INC  BC
    { &Z80::opcode_04,  4 }, // INC  B
    { &Z80::opcode_05,  4 }, // DEC  B
    { &Z80::opcode_06,  7 }, // LD   B,n
    { &Z80::opcode_07,  4 }, // RLCA
    { &Z80::opcode_08,  4 }, // EX   AF,AF'
    { &Z80::opcode_09, 11 }, // ADD  HL,BC
    { &Z80::opcode_0a,  7 }, // LD   A,(BC)
    { &Z80::opcode_0b,  6 }, // DEC  BC
    { &Z80::opcode_0c,  4 }, // INC  C
    { &Z80::opcode_0d,  4 }, // DEC  C
    { &Z80::opcode_0e,  7 }, // LD   C,n
    { &Z80::opcode_0f,  4 }, // RRCA
    { &Z80::opcode_10,  8 }, // DJNZ d
    { &Z80::opcode_11, 10 }, // LD   DE,nn
    { &Z80::opcode_12,  7 }, // LD   (DE),A
    { &Z80::opcode_13,  6 }, // INC  DE
    { &Z80::opcode_14,  4 }, // INC  D
    { &Z80::opcode_15,  4 }, // DEC  D
    { &Z80::opcode_16,  7 }, // LD   D,n
    { &Z80::opcode_17,  4 }, // RLA
    { &Z80::opcode_18, 12 }, // JR   d
    { &Z80::opcode_19, 11 }, // ADD  HL,DE
    { &Z80::opcode_1a,  7 }, // LD   A,(DE)
    { &Z80::opcode_1b,  6 }, // DEC  DE
    { &Z80::opcode_1c,  4 }, // INC  E
    { &Z80::opcode_1d,  4 }, // DEC  E
    { &Z80::opcode_1e,  7 }, // LD   E,n
    { &Z80::opcode_1f,  4 }, // RRA
    { &Z80::opcode_20,  7 }, // JR   NZ,d
    { &Z80::opcode_21, 10 }, // LD   HL,nn
    { &Z80::opcode_22, 16 }, // LD   (nn),HL
    { &Z80::opcode_23,  6 }, // INC  HL
    { &Z80::opcode_24,  4 }, // INC  H
    { &Z80::opcode_25,  4 }, // DEC  H
    { &Z80::opcode_26,  7 }, // LD   H,n
    { &Z80::opcode_27,  4 }, // DAA
    { &Z80::opcode_28,  7 }, // JR   Z,d
    { &Z80::opcode_29, 11 }, // ADD  HL,HL
    { &Z80::opcode_2a, 16 }, // LD   HL,(nn)
    { &Z80::opcode_2b,  6 }, // DEC  HL
    { &Z80::opcode_2c,  4 }, // INC  L
    { &Z80::opcode_2d,  4 }, // DEC  L
    { &Z80::opcode_2e,  7 }, // LD   L,n
    { &Z80::opcode_2f,  4 }, // CPL
    { &Z80::opcode_30,  7 }, // JR   NC,d
    { &Z80::opcode_31, 10 }, // LD   SP,nn
    { &Z80::opcode_32, 13 }, // LD   (nn),A
    { &Z80::opcode_33,  6 }, // INC  SP
    { &Z80::opcode_34, 11 }, // INC  (HL)
    { &Z80::opcode_35, 11 }, // DEC  (HL)
    { &Z80::opcode_36, 10 }, // LD   (HL),n
    { &Z80::opcode_37,  4 }, // SCF
    { &Z80::opcode_38,  7 }, // JR   C,d
    { &Z80::opcode_39, 11 }, // ADD  HL,SP
    { &Z80::opcode_3a, 13 }, // LD   A,(nn)
    { &Z80::opcode_3b,  6 }, // DEC  SP
    { &Z80::opcode_3c,  4 }, // INC  A
    { &Z80::opcode_3d,  4 }, // DEC  A
    { &Z80::opcode_3e,  7 }, // LD   A,n
    { &Z80::opcode_3f,  4 }, // CCF
    { &Z80::opcode_40,  4 }, // LD   B,B
    { &Z80::opcode_41,  4 }, // LD   B,C
    { &Z80::opcode_42,  4 }, // LD   B,D
    { &Z80::opcode_43,  4 }, // LD   B,E
    { &Z80::opcode_44,  4 }, // LD   B,H
    { &Z80::opcode_45,  4 }, // LD   B,L
    { &Z80::opcode_46,  7 }, // LD   B,(HL)
    { &Z80::opcode_47,  4 }, // LD   B,A
    { &Z80::opcode_48,  4 }, // LD   C,B
    { &Z80::opcode_49,  4 }, // LD   C,C
    { &Z80::opcode_4a,  4 }, // LD   C,D
    { &Z80::opcode_4b,  4 }, // LD   C,E
    { &Z80::opcode_4c,  4 }, // LD   C,H
    { &Z80::opcode_4d,  4 }, // LD   C,L
    { &Z80::opcode_4e,  7 }, // LD   C,(HL)
    { &Z80::opcode_4f,  4 }, // LD   C,A
    { &Z80::opcode_50,  4 }, // LD   D,B
    { &Z80::opcode_51,  4 }, // LD   D,C
    { &Z80::opcode_52,  4 }, // LD   D,D
    { &Z80::opcode_53,  4 }, // LD   D,E
    { &Z80::opcode_54,  4 }, // LD   D,H
    { &Z80::opcode_55,  4 }, // LD   D,L
    { &Z80::opcode_56,  7 }, // LD   D,(HL)
    { &Z80::opcode_57,  4 }, // LD   D,A
    { &Z80::opcode_58,  4 }, // LD   E,B
    { &Z80::opcode_59,  4 }, // LD   E,C
    { &Z80::opcode_5a,  4 }, // LD   E,D
    { &Z80::opcode_5b,  4 }, // LD   E,E
    { &Z80::opcode_5c,  4 }, // LD   E,H
    { &Z80::opcode_5d,  4 }, // LD   E,L
    { &Z80::opcode_5e,  7 }, // LD   E,(HL)
    { &Z80::opcode_5f,  4 }, // LD   E,A
    { &Z80::opcode_60,  4 }, // LD   H,B
    { &Z80::opcode_61,  4 }, // LD   H,C
    { &Z80::opcode_62,  4 }, // LD   H,D
    { &Z80::opcode_63,  4 }, // LD   H,E
    { &Z80::opcode_64,  4 }, // LD   H,H
    { &Z80::opcode_65,  4 }, // LD   H,L
    { &Z80::opcode_66,  7 }, // LD   H,(HL)
    { &Z80::opcode_67,  4 }, // LD   H,A
    { &Z80::opcode_68,  4 }, // LD   L,B
    { &Z80::opcode_69,  4 }, // LD   L,C
    { &Z80::opcode_6a,  4 }, // LD   L,D
    { &Z80::opcode_6b,  4 }, // LD   L,E
    { &Z80::opcode_6c,  4 }, // LD   L,H
    { &Z80::opcode_6d,  4 }, // LD   L,L
    { &Z80::opcode_6e,  7 }, // LD   L,(HL)
    { &Z80::opcode_6f,  4 }, // LD   L,A
    { &Z80::opcode_70,  7 }, // LD   (HL),B
    { &Z80::opcode_71,  7 }, // LD   (HL),C
    { &Z80::opcode_72,  7 }, // LD   (HL),D
    { &Z80::opcode_73,  7 }, // LD   (HL),E
    { &Z80::opcode_74,  7 }, // LD   (HL),H
    { &Z80::opcode_75,  7 }, // LD   (HL),L
    { &Z80::opcode_76,  4 }, // HALT
    { &Z80::opcode_77,  7 }, // LD   (HL),A
    { &Z80::opcode_78,  4 }, // LD   A,B
    { &Z80::opcode_79,  4 }, // LD   A,C
    { &Z80::opcode_7a,  4 }, // LD   A,D
    { &Z80::opcode_7b,  4 }, // LD   A,E
    { &Z80::opcode_7c,  4 }, // LD   A,H
    { &Z80::opcode_7d,  4 }, // LD   A,L
    { &Z80::opcode_7e,  7 }, // LD   A,(HL)
    { &Z80::opcode_7f,  4 }, // LD   A,A
    { &Z80::opcode_80,  4 }, // ADD  A,B
    { &Z80::opcode_81,  4 }, // ADD  A,C
    { &Z80::opcode_82,  4 }, // ADD  A,D
    { &Z80::opcode_83,  4 }, // ADD  A,E
    { &Z80::opcode_84,  4 }, // ADD  A,H
    { &Z80::opcode_85,  4 }, // ADD  A,L
    { &Z80::opcode_86,  7 }, // ADD  A,(HL)
    { &Z80::opcode_87,  4 }, // ADD  A,A
    { &Z80::opcode_88,  4 }, // ADC  A,B
    { &Z80::opcode_89,  4 }, // ADC  A,C
    { &Z80::opcode_8a,  4 }, // ADC  A,D
    { &Z80::opcode_8b,  4 }, // ADC  A,E
    { &Z80::opcode_8c,  4 }, // ADC  A,H
    { &Z80::opcode_8d,  4 }, // ADC  A,L
    { &Z80::opcode_8e,  7 }, // ADC  A,(HL)
    { &Z80::opcode_8f,  4 }, // ADC  A,A
    { &Z80::opcode_90,  4 }, // SUB  B
    { &Z80::opcode_91,  4 }, // SUB  C
    { &Z80::opcode_92,  4 }, // SUB  D
    { &Z80::opcode_93,  4 }, // SUB  E
    { &Z80::opcode_94,  4 }, // SUB  H
    { &Z80::opcode_95,  4 }, // SUB  L
    { &Z80::opcode_96,  7 }, // SUB  (HL)
    { &Z80::opcode_97,  4 }, // SUB  A
    { &Z80::opcode_98,  4 }, // SBC  A,B
    { &Z80::opcode_99,  4 }, // SBC  A,C
    { &Z80::opcode_9a,  4 }, // SBC  A,D
    { &Z80::opcode_9b,  4 }, // SBC  A,E
    { &Z80::opcode_9c,  4 }, // SBC  A,H
    { &Z80::opcode_9d,  4 }, // SBC  A,L
    { &Z80::opcode_9e,  7 }, // SBC  A,(HL)
    { &Z80::opcode_9f,  4 }, // SBC  A,A
    { &Z80::opcode_a0,  4 }, // AND  B
    { &Z80::opcode_a1,  4 }, // AND  C
    { &Z80::opcode_a2,  4 }, // AND  D
    { &Z80::opcode_a3,  4 }, // AND  E
    { &Z80::opcode_a4,  4 }, // AND  H
    { &Z80::opcode_a5,  4 }, // AND  L
    { &Z80::opcode_a6,  7 }, // AND  (HL)
    { &Z80::opcode_a7,  4 }, // AND  A
    { &Z80::opcode_a8,  4 }, // XOR  B
    { &Z80::opcode_a9,  4 }, // XOR  C
    { &Z80::opcode_aa,  4 }, // XOR  D
    { &Z80::opcode_ab,  4 }, // XOR  E
    { &Z80::opcode_ac,  4 }, // XOR  H
    { &Z80::opcode_ad,  4 }, // XOR  L
    { &Z80::opcode_ae,  7 }, // XOR  (HL)
    { &Z80::opcode_af,  4 }, // XOR  A
    { &Z80::opcode_b0,  4 }, // OR   B
    { &Z80::opcode_b1,  4 }, // OR   C
    { &Z80::opcode_b2,  4 }, // OR   D
    { &Z80::opcode_b3,  4 }, // OR   E
    { &Z80::opcode_b4,  4 }, // OR   H
    { &Z80::opcode_b5,  4 }, // OR   L
    { &Z80::opcode_b6,  7 }, // OR   (HL)
    { &Z80::opcode_b7,  4 }, // OR   A
    { &Z80::opcode_b8,  4 }, // CP   B
    { &Z80::opcode_b9,  4 }, // CP   C
    { &Z80::opcode_ba,  4 }, // CP   D
    { &Z80::opcode_bb,  4 }, // CP   E
    { &Z80::opcode_bc,  4 }, // CP   H
    { &Z80::opcode_bd,  4 }, // CP   L
    { &Z80::opcode_be,  7 }, // CP   (HL)
    { &Z80::opcode_bf,  4 }, // CP   A
    { &Z80::opcode_c0,  5 }, // RET  NZ
    { &Z80::opcode_c1, 10 }, // POP  BC
    { &Z80::opcode_c2, 10 }, // JP   NZ,nn
    { &Z80::opcode_c3, 10 }, // JP   nn
    { &Z80::opcode_c4, 10 }, // CALL NZ,nn
    { &Z80::opcode_c5, 11 }, // PUSH BC
    { &Z80::opcode_c6,  7 }, // ADD  A,n
    { &Z80::opcode_c7, 11 }, // RST  0
    { &Z80::opcode_c8,  5 }, // RET  Z
    { &Z80::opcode_c9, 10 }, // RET
    { &Z80::opcode_ca, 10 }, // JP   Z,nn
    { &Z80::opcode_cb,  0 }, // [Prefix]
    { &Z80::opcode_cc, 10 }, // CALL Z,nn
    { &Z80::opcode_cd, 17 }, // CALL nn
    { &Z80::opcode_ce,  7 }, // ADC  A,n
    { &Z80::opcode_cf, 11 }, // RST  8
    { &Z80::opcode_d0,  5 }, // RET  NC
    { &Z80::opcode_d1, 10 }, // POP  DE
    { &Z80::opcode_d2, 10 }, // JP   NC,nn
    { &Z80::opcode_d3, 11 }, // OUT  (n),A
    { &Z80::opcode_d4, 10 }, // CALL NC,nn
    { &Z80::opcode_d5, 11 }, // PUSH DE
    { &Z80::opcode_d6,  7 }, // SUB  n
    { &Z80::opcode_d7, 11 }, // RST  10H
    { &Z80::opcode_d8,  5 }, // RET  C
    { &Z80::opcode_d9,  4 }, // EXX
    { &Z80::opcode_da, 10 }, // JP   C,nn
    { &Z80::opcode_db, 11 }, // IN   A,(n)
    { &Z80::opcode_dc, 10 }, // CALL C,nn
    { &Z80::opcode_dd,  0 }, // [IX Prefix]
    { &Z80::opcode_de,  7 }, // SBC  A,n
    { &Z80::opcode_df, 11 }, // RST  18H
    { &Z80::opcode_e0,  5 }, // RET  PO
    { &Z80::opcode_e1, 10 }, // POP  HL
    { &Z80::opcode_e2, 10 }, // JP   PO,nn
    { &Z80::opcode_e3, 19 }, // EX   (SP),HL
    { &Z80::opcode_e4, 10 }, // CALL PO,nn
    { &Z80::opcode_e5, 11 }, // PUSH HL
    { &Z80::opcode_e6,  7 }, // AND  n
    { &Z80::opcode_e7, 11 }, // RST  20H
    { &Z80::opcode_e8,  5 }, // RET  PE
    { &Z80::opcode_e9,  4 }, // JP   (HL)
    { &Z80::opcode_ea, 10 }, // JP   PE,nn
    { &Z80::opcode_eb,  4 }, // EX   DE,HL
    { &Z80::opcode_ec, 10 }, // CALL PE,nn
    { &Z80::opcode_ed,  0 }, // [Prefix]
    { &Z80::opcode_ee,  7 }, // XOR  n
    { &Z80::opcode_ef, 11 }, // RST  28H
    { &Z80::opcode_f0,  5 }, // RET  P
    { &Z80::opcode_f1, 10 }, // POP  AF
    { &Z80::opcode_f2, 10 }, // JP   P,nn
    { &Z80::opcode_f3,  4 }, // DI
    { &Z80::opcode_f4, 10 }, // CALL P,nn
    { &Z80::opcode_f5, 11 }, // PUSH AF
    { &Z80::opcode_f6,  7 }, // OR   n
    { &Z80::opcode_f7, 11 }, // RST  30H
    { &Z80::opcode_f8,  5 }, // RET  M
    { &Z80::opcode_f9,  6 }, // LD   SP,HL
    { &Z80::opcode_fa, 10 }, // JP   M,nn
    { &Z80::opcode_fb,  4 }, // EI
    { &Z80::opcode_fc, 10 }, // CALL M,nn
    { &Z80::opcode_fd,  0 }, // [IY Prefix]
    { &Z80::opcode_fe,  7 }, // CP   n
    { &Z80::opcode_ff, 11 }  // RST  38H
};                          

static unsigned daa_table[2048] =
{
    0x0044, 0x0100, 0x0200, 0x0304, 0x0400, 0x0504, 0x0604, 0x0700, 
    0x0808, 0x090c, 0x1010, 0x1114, 0x1214, 0x1310, 0x1414, 0x1510, 
    0x1000, 0x1104, 0x1204, 0x1300, 0x1404, 0x1500, 0x1600, 0x1704, 
    0x180c, 0x1908, 0x2030, 0x2134, 0x2234, 0x2330, 0x2434, 0x2530, 
    0x2020, 0x2124, 0x2224, 0x2320, 0x2424, 0x2520, 0x2620, 0x2724, 
    0x282c, 0x2928, 0x3034, 0x3130, 0x3230, 0x3334, 0x3430, 0x3534, 
    0x3024, 0x3120, 0x3220, 0x3324, 0x3420, 0x3524, 0x3624, 0x3720, 
    0x3828, 0x392c, 0x4010, 0x4114, 0x4214, 0x4310, 0x4414, 0x4510, 
    0x4000, 0x4104, 0x4204, 0x4300, 0x4404, 0x4500, 0x4600, 0x4704, 
    0x480c, 0x4908, 0x5014, 0x5110, 0x5210, 0x5314, 0x5410, 0x5514, 
    0x5004, 0x5100, 0x5200, 0x5304, 0x5400, 0x5504, 0x5604, 0x5700, 
    0x5808, 0x590c, 0x6034, 0x6130, 0x6230, 0x6334, 0x6430, 0x6534, 
    0x6024, 0x6120, 0x6220, 0x6324, 0x6420, 0x6524, 0x6624, 0x6720, 
    0x6828, 0x692c, 0x7030, 0x7134, 0x7234, 0x7330, 0x7434, 0x7530, 
    0x7020, 0x7124, 0x7224, 0x7320, 0x7424, 0x7520, 0x7620, 0x7724, 
    0x782c, 0x7928, 0x8090, 0x8194, 0x8294, 0x8390, 0x8494, 0x8590, 
    0x8080, 0x8184, 0x8284, 0x8380, 0x8484, 0x8580, 0x8680, 0x8784, 
    0x888c, 0x8988, 0x9094, 0x9190, 0x9290, 0x9394, 0x9490, 0x9594, 
    0x9084, 0x9180, 0x9280, 0x9384, 0x9480, 0x9584, 0x9684, 0x9780, 
    0x9888, 0x998c, 0x0055, 0x0111, 0x0211, 0x0315, 0x0411, 0x0515, 
    0x0045, 0x0101, 0x0201, 0x0305, 0x0401, 0x0505, 0x0605, 0x0701, 
    0x0809, 0x090d, 0x1011, 0x1115, 0x1215, 0x1311, 0x1415, 0x1511, 
    0x1001, 0x1105, 0x1205, 0x1301, 0x1405, 0x1501, 0x1601, 0x1705, 
    0x180d, 0x1909, 0x2031, 0x2135, 0x2235, 0x2331, 0x2435, 0x2531, 
    0x2021, 0x2125, 0x2225, 0x2321, 0x2425, 0x2521, 0x2621, 0x2725, 
    0x282d, 0x2929, 0x3035, 0x3131, 0x3231, 0x3335, 0x3431, 0x3535, 
    0x3025, 0x3121, 0x3221, 0x3325, 0x3421, 0x3525, 0x3625, 0x3721, 
    0x3829, 0x392d, 0x4011, 0x4115, 0x4215, 0x4311, 0x4415, 0x4511, 
    0x4001, 0x4105, 0x4205, 0x4301, 0x4405, 0x4501, 0x4601, 0x4705, 
    0x480d, 0x4909, 0x5015, 0x5111, 0x5211, 0x5315, 0x5411, 0x5515, 
    0x5005, 0x5101, 0x5201, 0x5305, 0x5401, 0x5505, 0x5605, 0x5701, 
    0x5809, 0x590d, 0x6035, 0x6131, 0x6231, 0x6335, 0x6431, 0x6535, 
    0x6025, 0x6121, 0x6221, 0x6325, 0x6421, 0x6525, 0x6625, 0x6721, 
    0x6829, 0x692d, 0x7031, 0x7135, 0x7235, 0x7331, 0x7435, 0x7531, 
    0x7021, 0x7125, 0x7225, 0x7321, 0x7425, 0x7521, 0x7621, 0x7725, 
    0x782d, 0x7929, 0x8091, 0x8195, 0x8295, 0x8391, 0x8495, 0x8591, 
    0x8081, 0x8185, 0x8285, 0x8381, 0x8485, 0x8581, 0x8681, 0x8785, 
    0x888d, 0x8989, 0x9095, 0x9191, 0x9291, 0x9395, 0x9491, 0x9595, 
    0x9085, 0x9181, 0x9281, 0x9385, 0x9481, 0x9585, 0x9685, 0x9781, 
    0x9889, 0x998d, 0xa0b5, 0xa1b1, 0xa2b1, 0xa3b5, 0xa4b1, 0xa5b5, 
    0xa0a5, 0xa1a1, 0xa2a1, 0xa3a5, 0xa4a1, 0xa5a5, 0xa6a5, 0xa7a1, 
    0xa8a9, 0xa9ad, 0xb0b1, 0xb1b5, 0xb2b5, 0xb3b1, 0xb4b5, 0xb5b1, 
    0xb0a1, 0xb1a5, 0xb2a5, 0xb3a1, 0xb4a5, 0xb5a1, 0xb6a1, 0xb7a5, 
    0xb8ad, 0xb9a9, 0xc095, 0xc191, 0xc291, 0xc395, 0xc491, 0xc595, 
    0xc085, 0xc181, 0xc281, 0xc385, 0xc481, 0xc585, 0xc685, 0xc781, 
    0xc889, 0xc98d, 0xd091, 0xd195, 0xd295, 0xd391, 0xd495, 0xd591, 
    0xd081, 0xd185, 0xd285, 0xd381, 0xd485, 0xd581, 0xd681, 0xd785, 
    0xd88d, 0xd989, 0xe0b1, 0xe1b5, 0xe2b5, 0xe3b1, 0xe4b5, 0xe5b1, 
    0xe0a1, 0xe1a5, 0xe2a5, 0xe3a1, 0xe4a5, 0xe5a1, 0xe6a1, 0xe7a5, 
    0xe8ad, 0xe9a9, 0xf0b5, 0xf1b1, 0xf2b1, 0xf3b5, 0xf4b1, 0xf5b5, 
    0xf0a5, 0xf1a1, 0xf2a1, 0xf3a5, 0xf4a1, 0xf5a5, 0xf6a5, 0xf7a1, 
    0xf8a9, 0xf9ad, 0x0055, 0x0111, 0x0211, 0x0315, 0x0411, 0x0515, 
    0x0045, 0x0101, 0x0201, 0x0305, 0x0401, 0x0505, 0x0605, 0x0701, 
    0x0809, 0x090d, 0x1011, 0x1115, 0x1215, 0x1311, 0x1415, 0x1511, 
    0x1001, 0x1105, 0x1205, 0x1301, 0x1405, 0x1501, 0x1601, 0x1705, 
    0x180d, 0x1909, 0x2031, 0x2135, 0x2235, 0x2331, 0x2435, 0x2531, 
    0x2021, 0x2125, 0x2225, 0x2321, 0x2425, 0x2521, 0x2621, 0x2725, 
    0x282d, 0x2929, 0x3035, 0x3131, 0x3231, 0x3335, 0x3431, 0x3535, 
    0x3025, 0x3121, 0x3221, 0x3325, 0x3421, 0x3525, 0x3625, 0x3721, 
    0x3829, 0x392d, 0x4011, 0x4115, 0x4215, 0x4311, 0x4415, 0x4511, 
    0x4001, 0x4105, 0x4205, 0x4301, 0x4405, 0x4501, 0x4601, 0x4705, 
    0x480d, 0x4909, 0x5015, 0x5111, 0x5211, 0x5315, 0x5411, 0x5515, 
    0x5005, 0x5101, 0x5201, 0x5305, 0x5401, 0x5505, 0x5605, 0x5701, 
    0x5809, 0x590d, 0x6035, 0x6131, 0x6231, 0x6335, 0x6431, 0x6535, 
    0x0604, 0x0700, 0x0808, 0x090c, 0x0a0c, 0x0b08, 0x0c0c, 0x0d08, 
    0x0e08, 0x0f0c, 0x1010, 0x1114, 0x1214, 0x1310, 0x1414, 0x1510, 
    0x1600, 0x1704, 0x180c, 0x1908, 0x1a08, 0x1b0c, 0x1c08, 0x1d0c, 
    0x1e0c, 0x1f08, 0x2030, 0x2134, 0x2234, 0x2330, 0x2434, 0x2530, 
    0x2620, 0x2724, 0x282c, 0x2928, 0x2a28, 0x2b2c, 0x2c28, 0x2d2c, 
    0x2e2c, 0x2f28, 0x3034, 0x3130, 0x3230, 0x3334, 0x3430, 0x3534, 
    0x3624, 0x3720, 0x3828, 0x392c, 0x3a2c, 0x3b28, 0x3c2c, 0x3d28, 
    0x3e28, 0x3f2c, 0x4010, 0x4114, 0x4214, 0x4310, 0x4414, 0x4510, 
    0x4600, 0x4704, 0x480c, 0x4908, 0x4a08, 0x4b0c, 0x4c08, 0x4d0c, 
    0x4e0c, 0x4f08, 0x5014, 0x5110, 0x5210, 0x5314, 0x5410, 0x5514, 
    0x5604, 0x5700, 0x5808, 0x590c, 0x5a0c, 0x5b08, 0x5c0c, 0x5d08, 
    0x5e08, 0x5f0c, 0x6034, 0x6130, 0x6230, 0x6334, 0x6430, 0x6534, 
    0x6624, 0x6720, 0x6828, 0x692c, 0x6a2c, 0x6b28, 0x6c2c, 0x6d28, 
    0x6e28, 0x6f2c, 0x7030, 0x7134, 0x7234, 0x7330, 0x7434, 0x7530, 
    0x7620, 0x7724, 0x782c, 0x7928, 0x7a28, 0x7b2c, 0x7c28, 0x7d2c, 
    0x7e2c, 0x7f28, 0x8090, 0x8194, 0x8294, 0x8390, 0x8494, 0x8590, 
    0x8680, 0x8784, 0x888c, 0x8988, 0x8a88, 0x8b8c, 0x8c88, 0x8d8c, 
    0x8e8c, 0x8f88, 0x9094, 0x9190, 0x9290, 0x9394, 0x9490, 0x9594, 
    0x9684, 0x9780, 0x9888, 0x998c, 0x9a8c, 0x9b88, 0x9c8c, 0x9d88, 
    0x9e88, 0x9f8c, 0x0055, 0x0111, 0x0211, 0x0315, 0x0411, 0x0515, 
    0x0605, 0x0701, 0x0809, 0x090d, 0x0a0d, 0x0b09, 0x0c0d, 0x0d09, 
    0x0e09, 0x0f0d, 0x1011, 0x1115, 0x1215, 0x1311, 0x1415, 0x1511, 
    0x1601, 0x1705, 0x180d, 0x1909, 0x1a09, 0x1b0d, 0x1c09, 0x1d0d, 
    0x1e0d, 0x1f09, 0x2031, 0x2135, 0x2235, 0x2331, 0x2435, 0x2531, 
    0x2621, 0x2725, 0x282d, 0x2929, 0x2a29, 0x2b2d, 0x2c29, 0x2d2d, 
    0x2e2d, 0x2f29, 0x3035, 0x3131, 0x3231, 0x3335, 0x3431, 0x3535, 
    0x3625, 0x3721, 0x3829, 0x392d, 0x3a2d, 0x3b29, 0x3c2d, 0x3d29, 
    0x3e29, 0x3f2d, 0x4011, 0x4115, 0x4215, 0x4311, 0x4415, 0x4511, 
    0x4601, 0x4705, 0x480d, 0x4909, 0x4a09, 0x4b0d, 0x4c09, 0x4d0d, 
    0x4e0d, 0x4f09, 0x5015, 0x5111, 0x5211, 0x5315, 0x5411, 0x5515, 
    0x5605, 0x5701, 0x5809, 0x590d, 0x5a0d, 0x5b09, 0x5c0d, 0x5d09, 
    0x5e09, 0x5f0d, 0x6035, 0x6131, 0x6231, 0x6335, 0x6431, 0x6535, 
    0x6625, 0x6721, 0x6829, 0x692d, 0x6a2d, 0x6b29, 0x6c2d, 0x6d29, 
    0x6e29, 0x6f2d, 0x7031, 0x7135, 0x7235, 0x7331, 0x7435, 0x7531, 
    0x7621, 0x7725, 0x782d, 0x7929, 0x7a29, 0x7b2d, 0x7c29, 0x7d2d, 
    0x7e2d, 0x7f29, 0x8091, 0x8195, 0x8295, 0x8391, 0x8495, 0x8591, 
    0x8681, 0x8785, 0x888d, 0x8989, 0x8a89, 0x8b8d, 0x8c89, 0x8d8d, 
    0x8e8d, 0x8f89, 0x9095, 0x9191, 0x9291, 0x9395, 0x9491, 0x9595, 
    0x9685, 0x9781, 0x9889, 0x998d, 0x9a8d, 0x9b89, 0x9c8d, 0x9d89, 
    0x9e89, 0x9f8d, 0xa0b5, 0xa1b1, 0xa2b1, 0xa3b5, 0xa4b1, 0xa5b5, 
    0xa6a5, 0xa7a1, 0xa8a9, 0xa9ad, 0xaaad, 0xaba9, 0xacad, 0xada9, 
    0xaea9, 0xafad, 0xb0b1, 0xb1b5, 0xb2b5, 0xb3b1, 0xb4b5, 0xb5b1, 
    0xb6a1, 0xb7a5, 0xb8ad, 0xb9a9, 0xbaa9, 0xbbad, 0xbca9, 0xbdad, 
    0xbead, 0xbfa9, 0xc095, 0xc191, 0xc291, 0xc395, 0xc491, 0xc595, 
    0xc685, 0xc781, 0xc889, 0xc98d, 0xca8d, 0xcb89, 0xcc8d, 0xcd89, 
    0xce89, 0xcf8d, 0xd091, 0xd195, 0xd295, 0xd391, 0xd495, 0xd591, 
    0xd681, 0xd785, 0xd88d, 0xd989, 0xda89, 0xdb8d, 0xdc89, 0xdd8d, 
    0xde8d, 0xdf89, 0xe0b1, 0xe1b5, 0xe2b5, 0xe3b1, 0xe4b5, 0xe5b1, 
    0xe6a1, 0xe7a5, 0xe8ad, 0xe9a9, 0xeaa9, 0xebad, 0xeca9, 0xedad, 
    0xeead, 0xefa9, 0xf0b5, 0xf1b1, 0xf2b1, 0xf3b5, 0xf4b1, 0xf5b5, 
    0xf6a5, 0xf7a1, 0xf8a9, 0xf9ad, 0xfaad, 0xfba9, 0xfcad, 0xfda9, 
    0xfea9, 0xffad, 0x0055, 0x0111, 0x0211, 0x0315, 0x0411, 0x0515, 
    0x0605, 0x0701, 0x0809, 0x090d, 0x0a0d, 0x0b09, 0x0c0d, 0x0d09, 
    0x0e09, 0x0f0d, 0x1011, 0x1115, 0x1215, 0x1311, 0x1415, 0x1511, 
    0x1601, 0x1705, 0x180d, 0x1909, 0x1a09, 0x1b0d, 0x1c09, 0x1d0d, 
    0x1e0d, 0x1f09, 0x2031, 0x2135, 0x2235, 0x2331, 0x2435, 0x2531, 
    0x2621, 0x2725, 0x282d, 0x2929, 0x2a29, 0x2b2d, 0x2c29, 0x2d2d, 
    0x2e2d, 0x2f29, 0x3035, 0x3131, 0x3231, 0x3335, 0x3431, 0x3535, 
    0x3625, 0x3721, 0x3829, 0x392d, 0x3a2d, 0x3b29, 0x3c2d, 0x3d29, 
    0x3e29, 0x3f2d, 0x4011, 0x4115, 0x4215, 0x4311, 0x4415, 0x4511, 
    0x4601, 0x4705, 0x480d, 0x4909, 0x4a09, 0x4b0d, 0x4c09, 0x4d0d, 
    0x4e0d, 0x4f09, 0x5015, 0x5111, 0x5211, 0x5315, 0x5411, 0x5515, 
    0x5605, 0x5701, 0x5809, 0x590d, 0x5a0d, 0x5b09, 0x5c0d, 0x5d09, 
    0x5e09, 0x5f0d, 0x6035, 0x6131, 0x6231, 0x6335, 0x6431, 0x6535, 
    0x0046, 0x0102, 0x0202, 0x0306, 0x0402, 0x0506, 0x0606, 0x0702, 
    0x080a, 0x090e, 0x0402, 0x0506, 0x0606, 0x0702, 0x080a, 0x090e, 
    0x1002, 0x1106, 0x1206, 0x1302, 0x1406, 0x1502, 0x1602, 0x1706, 
    0x180e, 0x190a, 0x1406, 0x1502, 0x1602, 0x1706, 0x180e, 0x190a, 
    0x2022, 0x2126, 0x2226, 0x2322, 0x2426, 0x2522, 0x2622, 0x2726, 
    0x282e, 0x292a, 0x2426, 0x2522, 0x2622, 0x2726, 0x282e, 0x292a, 
    0x3026, 0x3122, 0x3222, 0x3326, 0x3422, 0x3526, 0x3626, 0x3722, 
    0x382a, 0x392e, 0x3422, 0x3526, 0x3626, 0x3722, 0x382a, 0x392e, 
    0x4002, 0x4106, 0x4206, 0x4302, 0x4406, 0x4502, 0x4602, 0x4706, 
    0x480e, 0x490a, 0x4406, 0x4502, 0x4602, 0x4706, 0x480e, 0x490a, 
    0x5006, 0x5102, 0x5202, 0x5306, 0x5402, 0x5506, 0x5606, 0x5702, 
    0x580a, 0x590e, 0x5402, 0x5506, 0x5606, 0x5702, 0x580a, 0x590e, 
    0x6026, 0x6122, 0x6222, 0x6326, 0x6422, 0x6526, 0x6626, 0x6722, 
    0x682a, 0x692e, 0x6422, 0x6526, 0x6626, 0x6722, 0x682a, 0x692e, 
    0x7022, 0x7126, 0x7226, 0x7322, 0x7426, 0x7522, 0x7622, 0x7726, 
    0x782e, 0x792a, 0x7426, 0x7522, 0x7622, 0x7726, 0x782e, 0x792a, 
    0x8082, 0x8186, 0x8286, 0x8382, 0x8486, 0x8582, 0x8682, 0x8786, 
    0x888e, 0x898a, 0x8486, 0x8582, 0x8682, 0x8786, 0x888e, 0x898a, 
    0x9086, 0x9182, 0x9282, 0x9386, 0x9482, 0x9586, 0x9686, 0x9782, 
    0x988a, 0x998e, 0x3423, 0x3527, 0x3627, 0x3723, 0x382b, 0x392f, 
    0x4003, 0x4107, 0x4207, 0x4303, 0x4407, 0x4503, 0x4603, 0x4707, 
    0x480f, 0x490b, 0x4407, 0x4503, 0x4603, 0x4707, 0x480f, 0x490b, 
    0x5007, 0x5103, 0x5203, 0x5307, 0x5403, 0x5507, 0x5607, 0x5703, 
    0x580b, 0x590f, 0x5403, 0x5507, 0x5607, 0x5703, 0x580b, 0x590f, 
    0x6027, 0x6123, 0x6223, 0x6327, 0x6423, 0x6527, 0x6627, 0x6723, 
    0x682b, 0x692f, 0x6423, 0x6527, 0x6627, 0x6723, 0x682b, 0x692f, 
    0x7023, 0x7127, 0x7227, 0x7323, 0x7427, 0x7523, 0x7623, 0x7727, 
    0x782f, 0x792b, 0x7427, 0x7523, 0x7623, 0x7727, 0x782f, 0x792b, 
    0x8083, 0x8187, 0x8287, 0x8383, 0x8487, 0x8583, 0x8683, 0x8787, 
    0x888f, 0x898b, 0x8487, 0x8583, 0x8683, 0x8787, 0x888f, 0x898b, 
    0x9087, 0x9183, 0x9283, 0x9387, 0x9483, 0x9587, 0x9687, 0x9783, 
    0x988b, 0x998f, 0x9483, 0x9587, 0x9687, 0x9783, 0x988b, 0x998f, 
    0xa0a7, 0xa1a3, 0xa2a3, 0xa3a7, 0xa4a3, 0xa5a7, 0xa6a7, 0xa7a3, 
    0xa8ab, 0xa9af, 0xa4a3, 0xa5a7, 0xa6a7, 0xa7a3, 0xa8ab, 0xa9af, 
    0xb0a3, 0xb1a7, 0xb2a7, 0xb3a3, 0xb4a7, 0xb5a3, 0xb6a3, 0xb7a7, 
    0xb8af, 0xb9ab, 0xb4a7, 0xb5a3, 0xb6a3, 0xb7a7, 0xb8af, 0xb9ab, 
    0xc087, 0xc183, 0xc283, 0xc387, 0xc483, 0xc587, 0xc687, 0xc783, 
    0xc88b, 0xc98f, 0xc483, 0xc587, 0xc687, 0xc783, 0xc88b, 0xc98f, 
    0xd083, 0xd187, 0xd287, 0xd383, 0xd487, 0xd583, 0xd683, 0xd787, 
    0xd88f, 0xd98b, 0xd487, 0xd583, 0xd683, 0xd787, 0xd88f, 0xd98b, 
    0xe0a3, 0xe1a7, 0xe2a7, 0xe3a3, 0xe4a7, 0xe5a3, 0xe6a3, 0xe7a7, 
    0xe8af, 0xe9ab, 0xe4a7, 0xe5a3, 0xe6a3, 0xe7a7, 0xe8af, 0xe9ab, 
    0xf0a7, 0xf1a3, 0xf2a3, 0xf3a7, 0xf4a3, 0xf5a7, 0xf6a7, 0xf7a3, 
    0xf8ab, 0xf9af, 0xf4a3, 0xf5a7, 0xf6a7, 0xf7a3, 0xf8ab, 0xf9af, 
    0x0047, 0x0103, 0x0203, 0x0307, 0x0403, 0x0507, 0x0607, 0x0703, 
    0x080b, 0x090f, 0x0403, 0x0507, 0x0607, 0x0703, 0x080b, 0x090f, 
    0x1003, 0x1107, 0x1207, 0x1303, 0x1407, 0x1503, 0x1603, 0x1707, 
    0x180f, 0x190b, 0x1407, 0x1503, 0x1603, 0x1707, 0x180f, 0x190b, 
    0x2023, 0x2127, 0x2227, 0x2323, 0x2427, 0x2523, 0x2623, 0x2727, 
    0x282f, 0x292b, 0x2427, 0x2523, 0x2623, 0x2727, 0x282f, 0x292b, 
    0x3027, 0x3123, 0x3223, 0x3327, 0x3423, 0x3527, 0x3627, 0x3723, 
    0x382b, 0x392f, 0x3423, 0x3527, 0x3627, 0x3723, 0x382b, 0x392f, 
    0x4003, 0x4107, 0x4207, 0x4303, 0x4407, 0x4503, 0x4603, 0x4707, 
    0x480f, 0x490b, 0x4407, 0x4503, 0x4603, 0x4707, 0x480f, 0x490b, 
    0x5007, 0x5103, 0x5203, 0x5307, 0x5403, 0x5507, 0x5607, 0x5703, 
    0x580b, 0x590f, 0x5403, 0x5507, 0x5607, 0x5703, 0x580b, 0x590f, 
    0x6027, 0x6123, 0x6223, 0x6327, 0x6423, 0x6527, 0x6627, 0x6723, 
    0x682b, 0x692f, 0x6423, 0x6527, 0x6627, 0x6723, 0x682b, 0x692f, 
    0x7023, 0x7127, 0x7227, 0x7323, 0x7427, 0x7523, 0x7623, 0x7727, 
    0x782f, 0x792b, 0x7427, 0x7523, 0x7623, 0x7727, 0x782f, 0x792b, 
    0x8083, 0x8187, 0x8287, 0x8383, 0x8487, 0x8583, 0x8683, 0x8787, 
    0x888f, 0x898b, 0x8487, 0x8583, 0x8683, 0x8787, 0x888f, 0x898b, 
    0x9087, 0x9183, 0x9283, 0x9387, 0x9483, 0x9587, 0x9687, 0x9783, 
    0x988b, 0x998f, 0x9483, 0x9587, 0x9687, 0x9783, 0x988b, 0x998f, 
    0xfabe, 0xfbba, 0xfcbe, 0xfdba, 0xfeba, 0xffbe, 0x0046, 0x0102, 
    0x0202, 0x0306, 0x0402, 0x0506, 0x0606, 0x0702, 0x080a, 0x090e, 
    0x0a1e, 0x0b1a, 0x0c1e, 0x0d1a, 0x0e1a, 0x0f1e, 0x1002, 0x1106, 
    0x1206, 0x1302, 0x1406, 0x1502, 0x1602, 0x1706, 0x180e, 0x190a, 
    0x1a1a, 0x1b1e, 0x1c1a, 0x1d1e, 0x1e1e, 0x1f1a, 0x2022, 0x2126, 
    0x2226, 0x2322, 0x2426, 0x2522, 0x2622, 0x2726, 0x282e, 0x292a, 
    0x2a3a, 0x2b3e, 0x2c3a, 0x2d3e, 0x2e3e, 0x2f3a, 0x3026, 0x3122, 
    0x3222, 0x3326, 0x3422, 0x3526, 0x3626, 0x3722, 0x382a, 0x392e, 
    0x3a3e, 0x3b3a, 0x3c3e, 0x3d3a, 0x3e3a, 0x3f3e, 0x4002, 0x4106, 
    0x4206, 0x4302, 0x4406, 0x4502, 0x4602, 0x4706, 0x480e, 0x490a, 
    0x4a1a, 0x4b1e, 0x4c1a, 0x4d1e, 0x4e1e, 0x4f1a, 0x5006, 0x5102, 
    0x5202, 0x5306, 0x5402, 0x5506, 0x5606, 0x5702, 0x580a, 0x590e, 
    0x5a1e, 0x5b1a, 0x5c1e, 0x5d1a, 0x5e1a, 0x5f1e, 0x6026, 0x6122, 
    0x6222, 0x6326, 0x6422, 0x6526, 0x6626, 0x6722, 0x682a, 0x692e, 
    0x6a3e, 0x6b3a, 0x6c3e, 0x6d3a, 0x6e3a, 0x6f3e, 0x7022, 0x7126, 
    0x7226, 0x7322, 0x7426, 0x7522, 0x7622, 0x7726, 0x782e, 0x792a, 
    0x7a3a, 0x7b3e, 0x7c3a, 0x7d3e, 0x7e3e, 0x7f3a, 0x8082, 0x8186, 
    0x8286, 0x8382, 0x8486, 0x8582, 0x8682, 0x8786, 0x888e, 0x898a, 
    0x8a9a, 0x8b9e, 0x8c9a, 0x8d9e, 0x8e9e, 0x8f9a, 0x9086, 0x9182, 
    0x9282, 0x9386, 0x3423, 0x3527, 0x3627, 0x3723, 0x382b, 0x392f, 
    0x3a3f, 0x3b3b, 0x3c3f, 0x3d3b, 0x3e3b, 0x3f3f, 0x4003, 0x4107, 
    0x4207, 0x4303, 0x4407, 0x4503, 0x4603, 0x4707, 0x480f, 0x490b, 
    0x4a1b, 0x4b1f, 0x4c1b, 0x4d1f, 0x4e1f, 0x4f1b, 0x5007, 0x5103, 
    0x5203, 0x5307, 0x5403, 0x5507, 0x5607, 0x5703, 0x580b, 0x590f, 
    0x5a1f, 0x5b1b, 0x5c1f, 0x5d1b, 0x5e1b, 0x5f1f, 0x6027, 0x6123, 
    0x6223, 0x6327, 0x6423, 0x6527, 0x6627, 0x6723, 0x682b, 0x692f, 
    0x6a3f, 0x6b3b, 0x6c3f, 0x6d3b, 0x6e3b, 0x6f3f, 0x7023, 0x7127, 
    0x7227, 0x7323, 0x7427, 0x7523, 0x7623, 0x7727, 0x782f, 0x792b, 
    0x7a3b, 0x7b3f, 0x7c3b, 0x7d3f, 0x7e3f, 0x7f3b, 0x8083, 0x8187, 
    0x8287, 0x8383, 0x8487, 0x8583, 0x8683, 0x8787, 0x888f, 0x898b, 
    0x8a9b, 0x8b9f, 0x8c9b, 0x8d9f, 0x8e9f, 0x8f9b, 0x9087, 0x9183, 
    0x9283, 0x9387, 0x9483, 0x9587, 0x9687, 0x9783, 0x988b, 0x998f, 
    0x9a9f, 0x9b9b, 0x9c9f, 0x9d9b, 0x9e9b, 0x9f9f, 0xa0a7, 0xa1a3, 
    0xa2a3, 0xa3a7, 0xa4a3, 0xa5a7, 0xa6a7, 0xa7a3, 0xa8ab, 0xa9af, 
    0xaabf, 0xabbb, 0xacbf, 0xadbb, 0xaebb, 0xafbf, 0xb0a3, 0xb1a7, 
    0xb2a7, 0xb3a3, 0xb4a7, 0xb5a3, 0xb6a3, 0xb7a7, 0xb8af, 0xb9ab, 
    0xbabb, 0xbbbf, 0xbcbb, 0xbdbf, 0xbebf, 0xbfbb, 0xc087, 0xc183, 
    0xc283, 0xc387, 0xc483, 0xc587, 0xc687, 0xc783, 0xc88b, 0xc98f, 
    0xca9f, 0xcb9b, 0xcc9f, 0xcd9b, 0xce9b, 0xcf9f, 0xd083, 0xd187, 
    0xd287, 0xd383, 0xd487, 0xd583, 0xd683, 0xd787, 0xd88f, 0xd98b, 
    0xda9b, 0xdb9f, 0xdc9b, 0xdd9f, 0xde9f, 0xdf9b, 0xe0a3, 0xe1a7, 
    0xe2a7, 0xe3a3, 0xe4a7, 0xe5a3, 0xe6a3, 0xe7a7, 0xe8af, 0xe9ab, 
    0xeabb, 0xebbf, 0xecbb, 0xedbf, 0xeebf, 0xefbb, 0xf0a7, 0xf1a3, 
    0xf2a3, 0xf3a7, 0xf4a3, 0xf5a7, 0xf6a7, 0xf7a3, 0xf8ab, 0xf9af, 
    0xfabf, 0xfbbb, 0xfcbf, 0xfdbb, 0xfebb, 0xffbf, 0x0047, 0x0103, 
    0x0203, 0x0307, 0x0403, 0x0507, 0x0607, 0x0703, 0x080b, 0x090f, 
    0x0a1f, 0x0b1b, 0x0c1f, 0x0d1b, 0x0e1b, 0x0f1f, 0x1003, 0x1107, 
    0x1207, 0x1303, 0x1407, 0x1503, 0x1603, 0x1707, 0x180f, 0x190b, 
    0x1a1b, 0x1b1f, 0x1c1b, 0x1d1f, 0x1e1f, 0x1f1b, 0x2023, 0x2127, 
    0x2227, 0x2323, 0x2427, 0x2523, 0x2623, 0x2727, 0x282f, 0x292b, 
    0x2a3b, 0x2b3f, 0x2c3b, 0x2d3f, 0x2e3f, 0x2f3b, 0x3027, 0x3123, 
    0x3223, 0x3327, 0x3423, 0x3527, 0x3627, 0x3723, 0x382b, 0x392f, 
    0x3a3f, 0x3b3b, 0x3c3f, 0x3d3b, 0x3e3b, 0x3f3f, 0x4003, 0x4107, 
    0x4207, 0x4303, 0x4407, 0x4503, 0x4603, 0x4707, 0x480f, 0x490b, 
    0x4a1b, 0x4b1f, 0x4c1b, 0x4d1f, 0x4e1f, 0x4f1b, 0x5007, 0x5103, 
    0x5203, 0x5307, 0x5403, 0x5507, 0x5607, 0x5703, 0x580b, 0x590f, 
    0x5a1f, 0x5b1b, 0x5c1f, 0x5d1b, 0x5e1b, 0x5f1f, 0x6027, 0x6123, 
    0x6223, 0x6327, 0x6423, 0x6527, 0x6627, 0x6723, 0x682b, 0x692f, 
    0x6a3f, 0x6b3b, 0x6c3f, 0x6d3b, 0x6e3b, 0x6f3f, 0x7023, 0x7127, 
    0x7227, 0x7323, 0x7427, 0x7523, 0x7623, 0x7727, 0x782f, 0x792b, 
    0x7a3b, 0x7b3f, 0x7c3b, 0x7d3f, 0x7e3f, 0x7f3b, 0x8083, 0x8187, 
    0x8287, 0x8383, 0x8487, 0x8583, 0x8683, 0x8787, 0x888f, 0x898b, 
    0x8a9b, 0x8b9f, 0x8c9b, 0x8d9f, 0x8e9f, 0x8f9b, 0x9087, 0x9183, 
    0x9283, 0x9387, 0x9483, 0x9587, 0x9687, 0x9783, 0x988b, 0x998f
};

void Z80::opcode_00()    // NOP
{
}

void Z80::opcode_01()    // LD   BC,nn
{
    C = fetchByte();
    B = fetchByte();
}

void Z80::opcode_02()    // LD   (BC),A
{
    env_.writeByte( BC(), A );
}

void Z80::opcode_03()    // INC  BC
{
    if( ++C == 0 ) ++B;
}

void Z80::opcode_04()    // INC  B
{
    B = incByte( B );
}

void Z80::opcode_05()    // DEC  B
{
    B = decByte( B );
}

void Z80::opcode_06()    // LD   B,n
{
    B = fetchByte();
}

void Z80::opcode_07()    // RLCA
{
    A = (A << 1) | (A >> 7);
    F = F & ~(AddSub | Halfcarry | Carry) | (A & (Flag3 | Flag5));
    if( A & 0x01 ) F |= Carry;
}

void Z80::opcode_08()    // EX   AF,AF'
{
    unsigned char x;

    x = A; A = A1; A1 = x;
    x = F; F = F1; F1 = x;
}

void Z80::opcode_09()    // ADD  HL,BC
{
    unsigned hl = HL();
    unsigned rp = BC();
    unsigned x  = hl + rp;

    F &= (Flag3 | Flag5 | Sign | Zero | Parity);
    if( x > 0xFFFF ) F |= Carry;
    if( ((hl & 0xFFF) + (rp & 0xFFF)) > 0xFFF ) F |= Halfcarry;

    L = x & 0xFF;
    H = (x >> 8) & 0xFF;
}

void Z80::opcode_0a()    // LD   A,(BC)
{
    A = env_.readByte( BC() );
}

void Z80::opcode_0b()    // DEC  BC
{
    if( C-- == 0 ) --B;
}

void Z80::opcode_0c()    // INC  C
{
    C = incByte( C );
}

void Z80::opcode_0d()    // DEC  C
{
    C = decByte( C );
}

void Z80::opcode_0e()    // LD   C,n
{
    C = fetchByte();
}

void Z80::opcode_0f()    // RRCA
{
    A = (A >> 1) | (A << 7);
    F = F & ~(AddSub | Halfcarry | Carry) | (A & (Flag3 | Flag5));
    if( A & 0x80 ) F |= Carry;
}

void Z80::opcode_10()    // DJNZ d
{
    unsigned char   o = fetchByte();
    
    if( --B != 0 ) relJump( o ); 
}

void Z80::opcode_11()    // LD   DE,nn
{
    E = fetchByte();
    D = fetchByte();
}

void Z80::opcode_12()    // LD   (DE),A
{
    env_.writeByte( DE(), A );
}

void Z80::opcode_13()    // INC  DE
{
    if( ++E == 0 ) ++D;
}

void Z80::opcode_14()    // INC  D
{
    D = incByte( D );
}

void Z80::opcode_15()    // DEC  D
{
    D = decByte( D );
}

void Z80::opcode_16()    // LD   D,n
{
    D = fetchByte();
}

void Z80::opcode_17()    // RLA
{
    unsigned char   a = A;

    A <<= 1;
    if( F & Carry ) A |= 0x01;
    F = F & ~(AddSub | Halfcarry | Carry) | (A & (Flag3 | Flag5));
    if( a & 0x80 ) F |= Carry;
}

void Z80::opcode_18()    // JR   d
{
    relJump( fetchByte() );
}

void Z80::opcode_19()    // ADD  HL,DE
{
    unsigned hl = HL();
    unsigned rp = DE();
    unsigned x  = hl + rp;

    F &= (Flag3 | Flag5 | Sign | Zero | Parity);
    if( x > 0xFFFF ) F |= Carry;
    if( ((hl & 0xFFF) + (rp & 0xFFF)) > 0xFFF ) F |= Halfcarry;

    L = x & 0xFF;
    H = (x >> 8) & 0xFF;
}

void Z80::opcode_1a()    // LD   A,(DE)
{
    A = env_.readByte( DE() );
}

void Z80::opcode_1b()    // DEC  DE
{
    if( E-- == 0 ) --D;
}

void Z80::opcode_1c()    // INC  E
{
    E = incByte( E );
}

void Z80::opcode_1d()    // DEC  E
{
    E = decByte( E );
}

void Z80::opcode_1e()    // LD   E,n
{
    E = fetchByte();
}

void Z80::opcode_1f()    // RRA
{
    unsigned char   a = A;

    A >>= 1;
    if( F & Carry ) A |= 0x80;
    F = F & ~(AddSub | Halfcarry | Carry) | (A & (Flag3 | Flag5));
    if( a & 0x01 ) F |= Carry;
}

void Z80::opcode_20()    // JR   NZ,d
{
    unsigned char   o = fetchByte();
    
    if( ! (F & Zero) ) relJump( o );
}

void Z80::opcode_21()    // LD   HL,nn
{
    L = fetchByte();
    H = fetchByte();
}

void Z80::opcode_22()    // LD   (nn),HL
{
    unsigned x = fetchWord();

    env_.writeByte( x  , L );
    env_.writeByte( x+1, H );
}

void Z80::opcode_23()    // INC  HL
{
    if( ++L == 0 ) ++H;
}

void Z80::opcode_24()    // INC  H
{
    H = incByte( H );
}

void Z80::opcode_25()    // DEC  H
{
    H = decByte( H );
}

void Z80::opcode_26()    // LD   H,n
{
    H = fetchByte();
}

void Z80::opcode_27()    // DAA
{
    unsigned        x = A;

    if( F & Carry ) x |= 0x100;
    if( F & Subtraction ) x |= 0x200;
    if( F & Halfcarry ) x |= 0x400;

    x = daa_table[ x ];

    F = (unsigned char)(x & 0xFF);
    A = x >> 8;
}

void Z80::opcode_28()    // JR   Z,d
{
    unsigned char   o = fetchByte();
    
    if( F & Zero ) relJump( o );
}

void Z80::opcode_29()    // ADD  HL,HL
{
    unsigned hl = HL();
    unsigned rp = hl;
    unsigned x  = hl + rp;

    F &= (Flag3 | Flag5 | Sign | Zero | Parity);
    if( x > 0xFFFF ) F |= Carry;
    if( ((hl & 0xFFF) + (rp & 0xFFF)) > 0xFFF ) F |= Halfcarry;

    L = x & 0xFF;
    H = (x >> 8) & 0xFF;
}

void Z80::opcode_2a()    // LD   HL,(nn)
{
    unsigned x = fetchWord();

    L = env_.readByte( x );
    H = env_.readByte( x+1 );
}

void Z80::opcode_2b()    // DEC  HL
{
    if( L-- == 0 ) --H;
}

void Z80::opcode_2c()    // INC  L
{
    L = incByte( L );
}

void Z80::opcode_2d()    // DEC  L
{
    L = decByte( L );
}

void Z80::opcode_2e()    // LD   L,n
{
    L = fetchByte();
}

void Z80::opcode_2f()    // CPL
{
    A ^= 0xFF;
    F = (F & ~(Flag3 | Flag5)) | AddSub | Halfcarry | (A & (Flag3 | Flag5));
}

void Z80::opcode_30()    // JR   NC,d
{
    unsigned char   o = fetchByte();
    
    if( ! (F & Carry) ) relJump( o );
}

void Z80::opcode_31()    // LD   SP,nn
{
    SP = fetchWord();
}

void Z80::opcode_32()    // LD   (nn),A
{
    env_.writeByte( fetchWord(), A );
}

void Z80::opcode_33()    // INC  SP
{
    SP = (SP + 1) & 0xFFFF;
}

void Z80::opcode_34()    // INC  (HL)
{
    env_.writeByte( HL(), incByte( env_.readByte( HL() ) ) );
}

void Z80::opcode_35()    // DEC  (HL)
{
    env_.writeByte( HL(), decByte( env_.readByte( HL() ) ) );
}

void Z80::opcode_36()    // LD   (HL),n
{
    env_.writeByte( HL(), fetchByte() );
}

void Z80::opcode_37()    // SCF
{
    F = (F & (Parity | Sign | Zero)) | Carry | (A & (Flag3 | Flag5));
}

void Z80::opcode_38()    // JR   C,d
{
    unsigned char   o = fetchByte();
    
    if( F & Carry ) relJump( o );
}

void Z80::opcode_39()    // ADD  HL,SP
{
    unsigned hl = HL();
    unsigned rp = SP;
    unsigned x  = hl + rp;

    F &= (Flag3 | Flag5 | Sign | Zero | Parity);
    if( x > 0xFFFF ) F |= Carry;
    if( ((hl & 0xFFF) + (rp & 0xFFF)) > 0xFFF ) F |= Halfcarry;

    L = x & 0xFF;
    H = (x >> 8) & 0xFF;
}

void Z80::opcode_3a()    // LD   A,(nn)
{
    A = env_.readByte( fetchWord() );
}

void Z80::opcode_3b()    // DEC  SP
{
    SP = (SP - 1) & 0xFFFF;
}

void Z80::opcode_3c()    // INC  A
{
    A = incByte( A );
}

void Z80::opcode_3d()    // DEC  A
{
    A = decByte( A );
}

void Z80::opcode_3e()    // LD   A,n
{
    A = fetchByte();
}

void Z80::opcode_3f()    // CCF
{
    F = (F & (Parity | Sign | Zero)) | ((F ^ Carry) & Carry) | (A & (Flag3 | Flag5));
    if( ! (F & Carry) ) F |= Halfcarry; // Halfcarry holds previous carry
}

void Z80::opcode_40()    // LD   B,B
{
}

void Z80::opcode_41()    // LD   B,C
{
    B = C;
}

void Z80::opcode_42()    // LD   B,D
{
    B = D;
}

void Z80::opcode_43()    // LD   B,E
{
    B = E;
}

void Z80::opcode_44()    // LD   B,H
{
    B = H;
}

void Z80::opcode_45()    // LD   B,L
{
    B = L;
}

void Z80::opcode_46()    // LD   B,(HL)
{
    B = env_.readByte( HL() );
}

void Z80::opcode_47()    // LD   B,A
{
    B = A;
}

void Z80::opcode_48()    // LD   C,B
{
    C = B;
}

void Z80::opcode_49()    // LD   C,C
{
}

void Z80::opcode_4a()    // LD   C,D
{
    C = D;
}

void Z80::opcode_4b()    // LD   C,E
{
    C = E;
}

void Z80::opcode_4c()    // LD   C,H
{
    C = H;
}

void Z80::opcode_4d()    // LD   C,L
{
    C = L;
}

void Z80::opcode_4e()    // LD   C,(HL)
{
    C = env_.readByte( HL() );
}

void Z80::opcode_4f()    // LD   C,A
{
    C = A;
}

void Z80::opcode_50()    // LD   D,B
{
    D = B;
}

void Z80::opcode_51()    // LD   D,C
{
    D = C;
}

void Z80::opcode_52()    // LD   D,D
{
}

void Z80::opcode_53()    // LD   D,E
{
    D = E;
}

void Z80::opcode_54()    // LD   D,H
{
    D = H;
}

void Z80::opcode_55()    // LD   D,L
{
    D = L;
}

void Z80::opcode_56()    // LD   D,(HL)
{
    D = env_.readByte( HL() );
}

void Z80::opcode_57()    // LD   D,A
{
    D = A;
}

void Z80::opcode_58()    // LD   E,B
{
    E = B;
}

void Z80::opcode_59()    // LD   E,C
{
    E = C;
}

void Z80::opcode_5a()    // LD   E,D
{
    E = D;
}

void Z80::opcode_5b()    // LD   E,E
{
}

void Z80::opcode_5c()    // LD   E,H
{
    E = H;
}

void Z80::opcode_5d()    // LD   E,L
{
    E = L;
}

void Z80::opcode_5e()    // LD   E,(HL)
{
    E = env_.readByte( HL() );
}

void Z80::opcode_5f()    // LD   E,A
{
    E = A;
}

void Z80::opcode_60()    // LD   H,B
{
    H = B;
}

void Z80::opcode_61()    // LD   H,C
{
    H = C;
}

void Z80::opcode_62()    // LD   H,D
{
    H = D;
}

void Z80::opcode_63()    // LD   H,E
{
    H = E;
}

void Z80::opcode_64()    // LD   H,H
{
}

void Z80::opcode_65()    // LD   H,L
{
    H = L;
}

void Z80::opcode_66()    // LD   H,(HL)
{
    H = env_.readByte( HL() );
}

void Z80::opcode_67()    // LD   H,A
{
    H = A;
}

void Z80::opcode_68()    // LD   L,B
{
    L = B;
}

void Z80::opcode_69()    // LD   L,C
{
    L = C;
}

void Z80::opcode_6a()    // LD   L,D
{
    L = D;
}

void Z80::opcode_6b()    // LD   L,E
{
    L = E;
}

void Z80::opcode_6c()    // LD   L,H
{
    L = H;
}

void Z80::opcode_6d()    // LD   L,L
{
}

void Z80::opcode_6e()    // LD   L,(HL)
{
    L = env_.readByte( HL() );
}

void Z80::opcode_6f()    // LD   L,A
{
    L = A;
}

void Z80::opcode_70()    // LD   (HL),B
{
    env_.writeByte( HL(), B );
}

void Z80::opcode_71()    // LD   (HL),C
{
    env_.writeByte( HL(), C );
}

void Z80::opcode_72()    // LD   (HL),D
{
    env_.writeByte( HL(), D );
}

void Z80::opcode_73()    // LD   (HL),E
{
    env_.writeByte( HL(), E );
}

void Z80::opcode_74()    // LD   (HL),H
{
    env_.writeByte( HL(), H );
}

void Z80::opcode_75()    // LD   (HL),L
{
    env_.writeByte( HL(), L );
}

void Z80::opcode_76()    // HALT
{
    PC--;
    iflags_ |= Halted;
}

void Z80::opcode_77()    // LD   (HL),A
{
    env_.writeByte( HL(), A );
}

void Z80::opcode_78()    // LD   A,B
{
    A = B;
}

void Z80::opcode_79()    // LD   A,C
{
    A = C;
}

void Z80::opcode_7a()    // LD   A,D
{
    A = D;
}

void Z80::opcode_7b()    // LD   A,E
{
    A = E;
}

void Z80::opcode_7c()    // LD   A,H
{
    A = H;
}

void Z80::opcode_7d()    // LD   A,L
{
    A = L;
}

void Z80::opcode_7e()    // LD   A,(HL)
{
    A = env_.readByte( HL() );
}

void Z80::opcode_7f()    // LD   A,A
{
}

void Z80::opcode_80()    // ADD  A,B
{
    addByte( B, 0 );
}

void Z80::opcode_81()    // ADD  A,C
{
    addByte( C, 0 );
}

void Z80::opcode_82()    // ADD  A,D
{
    addByte( D, 0 );
}

void Z80::opcode_83()    // ADD  A,E
{
    addByte( E, 0 );
}

void Z80::opcode_84()    // ADD  A,H
{
    addByte( H, 0 );
}

void Z80::opcode_85()    // ADD  A,L
{
    addByte( L, 0 );
}

void Z80::opcode_86()    // ADD  A,(HL)
{
    addByte( env_.readByte( HL() ), 0 );
}

void Z80::opcode_87()    // ADD  A,A
{
    addByte( A, 0 );
}

void Z80::opcode_88()    // ADC  A,B
{
    addByte( B, F & Carry );
}

void Z80::opcode_89()    // ADC  A,C
{
    addByte( C, F & Carry );
}

void Z80::opcode_8a()    // ADC  A,D
{
    addByte( D, F & Carry );
}

void Z80::opcode_8b()    // ADC  A,E
{
    addByte( E, F & Carry );
}

void Z80::opcode_8c()    // ADC  A,H
{
    addByte( H, F & Carry );
}

void Z80::opcode_8d()    // ADC  A,L
{
    addByte( L, F & Carry );
}

void Z80::opcode_8e()    // ADC  A,(HL)
{
    addByte( env_.readByte( HL() ), F & Carry );
}

void Z80::opcode_8f()    // ADC  A,A
{
    addByte( A, F & Carry );
}

void Z80::opcode_90()    // SUB  B
{
    A = subByte( B, 0 );
}

void Z80::opcode_91()    // SUB  C
{
    A = subByte( C, 0 );
}

void Z80::opcode_92()    // SUB  D
{
    A = subByte( D, 0 );
}

void Z80::opcode_93()    // SUB  E
{
    A = subByte( E, 0 );
}

void Z80::opcode_94()    // SUB  H
{
    A = subByte( H, 0 );
}

void Z80::opcode_95()    // SUB  L
{
    A = subByte( L, 0 );
}

void Z80::opcode_96()    // SUB  (HL)
{
    A = subByte( env_.readByte( HL() ), 0 );
}

void Z80::opcode_97()    // SUB  A
{
    A = subByte( A, 0 );
}

void Z80::opcode_98()    // SBC  A,B
{
    A = subByte( B, F & Carry );
}

void Z80::opcode_99()    // SBC  A,C
{
    A = subByte( C, F & Carry );
}

void Z80::opcode_9a()    // SBC  A,D
{
    A = subByte( D, F & Carry );
}

void Z80::opcode_9b()    // SBC  A,E
{
    A = subByte( E, F & Carry );
}

void Z80::opcode_9c()    // SBC  A,H
{
    A = subByte( H, F & Carry );
}

void Z80::opcode_9d()    // SBC  A,L
{
    A = subByte( L, F & Carry );
}

void Z80::opcode_9e()    // SBC  A,(HL)
{
    A = subByte( env_.readByte( HL() ), F & Carry );
}

void Z80::opcode_9f()    // SBC  A,A
{
    A = subByte( A, F & Carry );
}

void Z80::opcode_a0()    // AND  B
{
    A &= B;
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a1()    // AND  C
{
    A &= C;
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a2()    // AND  D
{
    A &= D;
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a3()    // AND  E
{
    A &= E;
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a4()    // AND  H
{
    A &= H;
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a5()    // AND  L
{
    A &= L;
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a6()    // AND  (HL)
{
    A &= env_.readByte( HL() );
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a7()    // AND  A
{
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_a8()    // XOR  B
{
    A ^= B;
    setFlags35PSZ000();
}

void Z80::opcode_a9()    // XOR  C
{
    A ^= C;
    setFlags35PSZ000();
}

void Z80::opcode_aa()    // XOR  D
{
    A ^= D;
    setFlags35PSZ000();
}

void Z80::opcode_ab()    // XOR  E
{
    A ^= E;
    setFlags35PSZ000();
}

void Z80::opcode_ac()    // XOR  H
{
    A ^= H;
    setFlags35PSZ000();
}

void Z80::opcode_ad()    // XOR  L
{
    A ^= L;
    setFlags35PSZ000();
}

void Z80::opcode_ae()    // XOR  (HL)
{
    A ^= env_.readByte( HL() );
    setFlags35PSZ000();
}

void Z80::opcode_af()    // XOR  A
{
    A = 0;
    setFlags35PSZ000();
}

void Z80::opcode_b0()    // OR   B
{
    A |= B;
    setFlags35PSZ000();
}

void Z80::opcode_b1()    // OR   C
{
    A |= C;
    setFlags35PSZ000();
}

void Z80::opcode_b2()    // OR   D
{
    A |= D;
    setFlags35PSZ000();
}

void Z80::opcode_b3()    // OR   E
{
    A |= E;
    setFlags35PSZ000();
}

void Z80::opcode_b4()    // OR   H
{
    A |= H;
    setFlags35PSZ000();
}

void Z80::opcode_b5()    // OR   L
{
    A |= L;
    setFlags35PSZ000();
}

void Z80::opcode_b6()    // OR   (HL)
{
    A |= env_.readByte( HL() );
    setFlags35PSZ000();
}

void Z80::opcode_b7()    // OR   A
{
    setFlags35PSZ000();
}

void Z80::opcode_b8()    // CP   B
{
    cmpByte( B );
}

void Z80::opcode_b9()    // CP   C
{
    cmpByte( C );
}

void Z80::opcode_ba()    // CP   D
{
    cmpByte( D );
}

void Z80::opcode_bb()    // CP   E
{
    cmpByte( E );
}

void Z80::opcode_bc()    // CP   H
{
    cmpByte( H );
}

void Z80::opcode_bd()    // CP   L
{
    cmpByte( L );
}

void Z80::opcode_be()    // CP   (HL)
{
    cmpByte( env_.readByte( HL() ) );
}

void Z80::opcode_bf()    // CP   A
{
    cmpByte( A );
}

void Z80::opcode_c0()    // RET  NZ
{
    if( ! (F & Zero) ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_c1()    // POP  BC
{
    C = env_.readByte( SP++ );
    B = env_.readByte( SP++ );
}

void Z80::opcode_c2()    // JP   NZ,nn
{
    unsigned    pc = fetchWord();

    if( ! (F & Zero) )
        PC = pc;
}

void Z80::opcode_c3()    // JP   nn
{
     PC = env_.readWord( PC );
}

void Z80::opcode_c4()    // CALL NZ,nn
{
    unsigned    pc = fetchWord();

    if( ! (F & Zero) ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_c5()    // PUSH BC
{
    env_.writeByte( --SP, B );
    env_.writeByte( --SP, C );
}

void Z80::opcode_c6()    // ADD  A,n
{
    addByte( fetchByte(), 0 );
}

void Z80::opcode_c7()    // RST  0
{
    callSub( 0x00 );
}

void Z80::opcode_c8()    // RET  Z
{
    if( F & Zero ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_c9()    // RET
{
     retFromSub();
}

void Z80::opcode_ca()    // JP   Z,nn
{
    unsigned    pc = fetchWord();

     if( F & Zero )
        PC = pc;
}

void Z80::opcode_cb()    // [Prefix]
{
    unsigned    op = fetchByte();

    cycles_ += OpInfoCB_[ op ].cycles;
    (this->*(OpInfoCB_[ op ].handler))();
}

void Z80::opcode_cc()    // CALL Z,nn
{
    unsigned    pc = fetchWord();

    if( F & Zero ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_cd()    // CALL nn
{
    callSub( fetchWord() );
}

void Z80::opcode_ce()    // ADC  A,n
{
    addByte( fetchByte(), F & Carry );
}

void Z80::opcode_cf()    // RST  8
{
    callSub( 0x08 );
}

void Z80::opcode_d0()    // RET  NC
{
    if( ! (F & Carry) ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_d1()    // POP  DE
{
    E = env_.readByte( SP++ );
    D = env_.readByte( SP++ );
}

void Z80::opcode_d2()    // JP   NC,nn
{
    unsigned    pc = fetchWord();

    if( ! (F & Carry) )
        PC = pc;
}

void Z80::opcode_d3()    // OUT  (n),A
{
    env_.writePort( fetchByte(), A );
}

void Z80::opcode_d4()    // CALL NC,nn
{
    unsigned    pc = fetchWord();

    if( ! (F & Carry) ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_d5()    // PUSH DE
{
    env_.writeByte( --SP, D );
    env_.writeByte( --SP, E );
}

void Z80::opcode_d6()    // SUB  n
{
    A = subByte( fetchByte(), 0 );
}

void Z80::opcode_d7()    // RST  10H
{
    callSub( 0x10 );
}

void Z80::opcode_d8()    // RET  C
{
    if( F & Carry ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_d9()    // EXX
{
    unsigned char x;

    x = B; B = B1; B1 = x;
    x = C; C = C1; C1 = x;
    x = D; D = D1; D1 = x;
    x = E; E = E1; E1 = x;
    x = H; H = H1; H1 = x;
    x = L; L = L1; L1 = x;
}

void Z80::opcode_da()    // JP   C,nn
{
    unsigned    pc = fetchWord();

     if( F & Carry )
        PC = pc;
}

void Z80::opcode_db()    // IN   A,(n)
{
    A = env_.readPort( fetchByte() );
}

void Z80::opcode_dc()    // CALL C,nn
{
    unsigned    pc = fetchWord();

    if( F & Carry ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_dd()    // [IX Prefix]
{
    do_opcode_xy( OpInfoDD_ );
    IX &= 0xFFFF;
}

void Z80::opcode_de()    // SBC  A,n
{
    A = subByte( fetchByte(), F & Carry );
}

void Z80::opcode_df()    // RST  18H
{
    callSub( 0x18 );
}

void Z80::opcode_e0()    // RET  PO
{
    if( ! (F & Parity) ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_e1()    // POP  HL
{
    L = env_.readByte( SP++ );
    H = env_.readByte( SP++ );
}

void Z80::opcode_e2()    // JP   PO,nn
{
    unsigned    pc = fetchWord();

     if( ! (F & Parity) )
        PC = pc;
}

void Z80::opcode_e3()    // EX   (SP),HL
{
    unsigned char   x;

    x = env_.readByte( SP   ); env_.writeByte( SP,   L ); L = x;
    x = env_.readByte( SP+1 ); env_.writeByte( SP+1, H ); H = x;
}

void Z80::opcode_e4()    // CALL PO,nn
{
    unsigned    pc = fetchWord();

    if( ! (F & Parity) ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_e5()    // PUSH HL
{
    env_.writeByte( --SP, H );
    env_.writeByte( --SP, L );
}

void Z80::opcode_e6()    // AND  n
{
    A &= fetchByte();
    setFlags35PSZ000();
    F |= Halfcarry;
}

void Z80::opcode_e7()    // RST  20H
{
    callSub( 0x20 );
}

void Z80::opcode_e8()    // RET  PE
{
    if( F & Parity ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_e9()    // JP   (HL)
{
    PC = HL();
}

void Z80::opcode_ea()    // JP   PE,nn
{
    unsigned    pc = fetchWord();

    if( F & Parity )
        PC = pc;
}

void Z80::opcode_eb()    // EX   DE,HL
{
    unsigned char x;

    x = D; D = H; H = x;
    x = E; E = L; L = x;
}

void Z80::opcode_ec()    // CALL PE,nn
{
    unsigned    pc = fetchWord();

    if( F & Parity ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_ed()    // [Prefix]
{
    unsigned    op = fetchByte();

    cycles_ += OpInfoED_[ op ].cycles;
    if( OpInfoED_[ op ].handler ) {
        (this->*(OpInfoED_[ op ].handler))();
    }
}

void Z80::opcode_ee()    // XOR  n
{
    A ^= fetchByte();
    setFlags35PSZ000();
}

void Z80::opcode_ef()    // RST  28H
{
    callSub( 0x28 );
}

void Z80::opcode_f0()    // RET  P
{
    if( ! (F & Sign) ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_f1()    // POP  AF
{
    F = env_.readByte( SP++ );
    A = env_.readByte( SP++ );
}

void Z80::opcode_f2()    // JP   P,nn
{
    unsigned    pc = fetchWord();

    if( ! (F & Sign) )
        PC = pc;
}

void Z80::opcode_f3()    // DI
{
    iflags_ &= ~(IFF1 | IFF2);
}

void Z80::opcode_f4()    // CALL P,nn
{
    unsigned    pc = fetchWord();

    if( ! (F & Sign) ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_f5()    // PUSH AF
{
    env_.writeByte( --SP, A );
    env_.writeByte( --SP, F );
}

void Z80::opcode_f6()    // OR   n
{
    A |= fetchByte();
    setFlags35PSZ000();
}

void Z80::opcode_f7()    // RST  30H
{
    callSub( 0x30 );
}

void Z80::opcode_f8()    // RET  M
{
    if( F & Sign ) {
        retFromSub();
        cycles_ += 2;
    }
}

void Z80::opcode_f9()    // LD   SP,HL
{
    SP = HL();
}

void Z80::opcode_fa()    // JP   M,nn
{
    unsigned    pc = fetchWord();

    if( F & Sign )
        PC = pc;
}

void Z80::opcode_fb()    // EI
{
    iflags_ |= IFF1 | IFF2;
}

void Z80::opcode_fc()    // CALL M,nn
{
    unsigned    pc = fetchWord();

    if( F & Sign ) {
        callSub( pc );
        cycles_ += 2;
    }
}

void Z80::opcode_fd()    // [IY Prefix]
{
    do_opcode_xy( OpInfoFD_ );
    IY &= 0xFFFF;
}


void Z80::opcode_fe()    // CP   n
{
    subByte( fetchByte(), 0 );
}

void Z80::opcode_ff()    // RST  38H
{
    callSub( 0x38 );
}

void Z80::do_opcode_xy( OpcodeInfo * info )
{
    unsigned    op = fetchByte();

    if( (op == 0xDD) || (op == 0xFD) ) {
        // Exit now, to avoid possible infinite loops
        PC--;
        cycles_ += 1;
    }
    else if( op == 0xED ) {
        // IX or IY prefix is ignored for this opcode
        opcode_ed();
    }
    else {
        // Handle IX or IY prefix if possible
        if( info[ op ].handler ) {
            // Extended opcode is valid
            cycles_ += info[ op ].cycles;
            (this->*(info[ op ].handler))();
        }
        else {
            // Extended opcode not valid, fall back to standard opcode
            cycles_ += OpInfo_[ op ].cycles;
            (this->*(OpInfo_[ op ].handler))();
        }
    }
}
